@@ -1,14 +1,19 @@
-// Copyright (c) 2014-2016 Josh Blum
+// Copyright (c) 2014-2018 Josh Blum
 // SPDX-License-Identifier: BSL-1.0
 
 #include <SoapySDR/Version.hpp>
 #include <SoapySDR/Modules.hpp>
 #include <SoapySDR/Registry.hpp>
 #include <SoapySDR/Device.hpp>
+#include <SoapySDR/ConverterRegistry.hpp>
+#include <algorithm> //sort, min, max
 #include <cstdlib>
 #include <cstddef>
 #include <iostream>
+#include <iomanip>
 #include <getopt.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 std::string SoapySDRDeviceProbe(SoapySDR::Device *);
 int SoapySDRRateTest(
@@ -17,6 +22,17 @@ int SoapySDRRateTest(
     const std::string &channelStr,
     const std::string &directionStr);
 
+/***********************************************************************
+ * Print the banner
+ **********************************************************************/
+static void printBanner(void)
+{
+    std::cout << "######################################################" << std::endl;
+    std::cout << "##     Soapy SDR -- the SDR abstraction library     ##" << std::endl;
+    std::cout << "######################################################" << std::endl;
+    std::cout << std::endl;
+}
+
 /***********************************************************************
  * Print help message
  **********************************************************************/
@@ -29,7 +45,11 @@ static int printHelp(void)
     std::cout << "    --find[=\"driver=foo,type=bar\"] \t Discover available devices" << std::endl;
     std::cout << "    --make[=\"driver=foo,type=bar\"] \t Create a device instance" << std::endl;
     std::cout << "    --probe[=\"driver=foo,type=bar\"] \t Print detailed information" << std::endl;
+    std::cout << std::endl;
+
+    std::cout << "  Advanced options:" << std::endl;
     std::cout << "    --check[=driverName] \t\t Check if driver is present" << std::endl;
+    std::cout << "    --sparse             \t\t Simplified output for --find" << std::endl;
     std::cout << std::endl;
 
     std::cout << "  Rate testing options:" << std::endl;
@@ -51,60 +71,103 @@ static int printInfo(void)
     std::cout << "ABI Version: v" << SoapySDR::getABIVersion() << std::endl;
     std::cout << "Install root: " << SoapySDR::getRootPath() << std::endl;
 
-    for (const auto &path : SoapySDR::listSearchPaths())
-        std::cout << "Search path: " << path << std::endl;
-
+    //max path length for alignment
+    size_t maxPathLen(0);
+    const auto searchPaths = SoapySDR::listSearchPaths();
     const auto modules = SoapySDR::listModules();
-    for (const auto &mod : modules) std::cout << "Module found: " << mod << std::endl;
+    for (const auto &path : searchPaths) maxPathLen = std::max(maxPathLen, path.size());
+    for (const auto &mod : modules) maxPathLen = std::max(maxPathLen, mod.size());
+
+    //print search path information
+    for (const auto &path : searchPaths)
+    {
+        struct stat info;
+        const bool missing = (stat(path.c_str(), &info) != 0);
+        std::cout << "Search path:  " << path;
+        if (missing) std::cout << std::string(maxPathLen-path.size(), ' ') << " (missing)";
+        std::cout << std::endl;
+    }
+
+    //load each module and print information
+    for (const auto &mod : modules)
+    {
+        std::cout << "Module found: " << mod;
+        const auto &errMsg = SoapySDR::loadModule(mod);
+        if (not errMsg.empty()) std::cout << "\n  " << errMsg;
+        const auto version = SoapySDR::getModuleVersion(mod);
+        if (not version.empty()) std::cout << std::string(maxPathLen-mod.size(), ' ') << " (" << version << ")";
+        std::cout << std::endl;
+    }
     if (modules.empty()) std::cout << "No modules found!" << std::endl;
 
-    std::cout << "Loading modules... " << std::flush;
-    SoapySDR::loadModules();
-    std::cout << "done" << std::endl;
+    std::cout << "Available factories... ";
+    std::string factories;
+    for (const auto &it : SoapySDR::Registry::listFindFunctions())
+    {
+        if (not factories.empty()) factories += ", ";
+        factories += it.first;
+    }
+    if (factories.empty()) factories = "No factories found!";
+    std::cout << factories << std::endl;
+
+    std::cout << "Available converters..." << std::endl;
+    for (const auto &source : SoapySDR::ConverterRegistry::listAvailableSourceFormats())
+    {
+        std::string targets;
+        for (const auto &target : SoapySDR::ConverterRegistry::listTargetFormats(source))
+        {
+            if (not targets.empty()) targets += ", ";
+            targets += target;
+        }
+        std::cout << " - " << std::setw(5) << source << " -> [" << targets << "]" << std::endl;
+    }
 
-    std::cout << "Available factories...";
-    const auto factories = SoapySDR::Registry::listFindFunctions();
-    for (const auto &it : factories) std::cout << it.first << ", ";
-    if (factories.empty()) std::cout << "No factories found!" << std::endl;
-    std::cout << std::endl;
     return EXIT_SUCCESS;
 }
 
 /***********************************************************************
  * Find devices and print args
  **********************************************************************/
-static int findDevices(void)
+static int findDevices(const std::string &argStr, const bool sparse)
 {
-    std::string argStr;
-    if (optarg != NULL) argStr = optarg;
-
     const auto results = SoapySDR::Device::enumerate(argStr);
-    for (size_t i = 0; i < results.size(); i++)
+    if (sparse)
     {
-        std::cout << "Found device " << i << std::endl;
-        for (const auto &it : results[i])
+        std::vector<std::string> sparseResults;
+        for (size_t i = 0; i < results.size(); i++)
         {
-            std::cout << "  " << it.first << " = " << it.second << std::endl;
+            const auto it = results[i].find("label");
+            if (it != results[i].end()) sparseResults.push_back(it->second);
+            else sparseResults.push_back(SoapySDR::KwargsToString(results[i]));
+        }
+        std::sort(sparseResults.begin(), sparseResults.end());
+        for (size_t i = 0; i < sparseResults.size(); i++)
+        {
+            std::cout << i << ": " << sparseResults[i] << std::endl;
         }
-        std::cout << std::endl;
     }
-    if (results.empty())
+    else
     {
-        std::cerr << "No devices found!" << std::endl;
-        return EXIT_FAILURE;
+        for (size_t i = 0; i < results.size(); i++)
+        {
+            std::cout << "Found device " << i << std::endl;
+            for (const auto &it : results[i])
+            {
+                std::cout << "  " << it.first << " = " << it.second << std::endl;
+            }
+            std::cout << std::endl;
+        }
+        if (results.empty()) std::cerr << "No devices found!" << std::endl;
+        else std::cout << std::endl;
     }
-    std::cout << std::endl;
-    return EXIT_SUCCESS;
+    return results.empty()?EXIT_FAILURE:EXIT_SUCCESS;
 }
 
 /***********************************************************************
  * Make device and print hardware info
  **********************************************************************/
-static int makeDevice(void)
+static int makeDevice(const std::string &argStr)
 {
-    std::string argStr;
-    if (optarg != NULL) argStr = optarg;
-
     std::cout << "Make device " << argStr << std::endl;
     try
     {
@@ -129,11 +192,8 @@ static int makeDevice(void)
 /***********************************************************************
  * Make device and print detailed info
  **********************************************************************/
-static int probeDevice(void)
+static int probeDevice(const std::string &argStr)
 {
-    std::string argStr;
-    if (optarg != NULL) argStr = optarg;
-
     std::cout << "Probe device " << argStr << std::endl;
     try
     {
@@ -153,11 +213,8 @@ static int probeDevice(void)
 /***********************************************************************
  * Check the registry for a specific driver
  **********************************************************************/
-static int checkDriver(void)
+static int checkDriver(const std::string &driverName)
 {
-    std::string driverName;
-    if (optarg != NULL) driverName = optarg;
-
     std::cout << "Loading modules... " << std::flush;
     SoapySDR::loadModules();
     std::cout << "done" << std::endl;
@@ -182,15 +239,15 @@ static int checkDriver(void)
  **********************************************************************/
 int main(int argc, char *argv[])
 {
-    std::cout << "######################################################" << std::endl;
-    std::cout << "## Soapy SDR -- the SDR abstraction library" << std::endl;
-    std::cout << "######################################################" << std::endl;
-    std::cout << std::endl;
-
     std::string argStr;
     std::string chanStr;
     std::string dirStr;
     double sampleRate(0.0);
+    std::string driverName;
+    bool findDevicesFlag(false);
+    bool sparsePrintFlag(false);
+    bool makeDeviceFlag(false);
+    bool probeDeviceFlag(false);
 
     /*******************************************************************
      * parse command line options
@@ -201,7 +258,9 @@ int main(int argc, char *argv[])
         {"make", optional_argument, 0, 'm'},
         {"info", optional_argument, 0, 'i'},
         {"probe", optional_argument, 0, 'p'},
+
         {"check", optional_argument, 0, 'c'},
+        {"sparse", no_argument, 0, 's'},
 
         {"args", optional_argument, 0, 'a'},
         {"rate", optional_argument, 0, 'r'},
@@ -215,12 +274,30 @@ int main(int argc, char *argv[])
     {
         switch (option)
         {
-        case 'h': return printHelp();
-        case 'i': return printInfo();
-        case 'f': return findDevices();
-        case 'm': return makeDevice();
-        case 'p': return probeDevice();
-        case 'c': return checkDriver();
+        case 'h':
+            printBanner();
+            return printHelp();
+        case 'i':
+            printBanner();
+            return printInfo();
+        case 'f':
+            findDevicesFlag = true;
+            if (optarg != nullptr) argStr = optarg;
+            break;
+        case 'm':
+            makeDeviceFlag = true;
+            if (optarg != nullptr) argStr = optarg;
+            break;
+        case 'p':
+            probeDeviceFlag = true;
+            if (optarg != nullptr) argStr = optarg;
+            break;
+        case 'c':
+            if (optarg != nullptr) driverName = optarg;
+            break;
+        case 's':
+            sparsePrintFlag = true;
+            break;
         case 'a':
             if (optarg != nullptr) argStr = optarg;
             break;
@@ -236,6 +313,12 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (not sparsePrintFlag) printBanner();
+    if (not driverName.empty()) return checkDriver(driverName);
+    if (findDevicesFlag) return findDevices(argStr, sparsePrintFlag);
+    if (makeDeviceFlag)  return makeDevice(argStr);
+    if (probeDeviceFlag) return probeDevice(argStr);
+
     //invoke utilities that rely on multiple arguments
     if (sampleRate != 0.0)
     {
