@@ -1,11 +1,11 @@
-// Copyright (c) 2014-2016 Josh Blum
+// Copyright (c) 2014-2017 Josh Blum
 // Copyright (c) 2016-2016 Bastille Networks
 // SPDX-License-Identifier: BSL-1.0
 
 #include <SoapySDR/Device.hpp>
 #include <SoapySDR/Formats.hpp>
 #include <cstdlib>
-#include <algorithm> //min/max
+#include <algorithm> //min/max/find
 
 SoapySDR::Device::~Device(void)
 {
@@ -218,6 +218,34 @@ std::complex<double> SoapySDR::Device::getIQBalance(const int, const size_t) con
     return std::complex<double>();
 }
 
+bool SoapySDR::Device::hasFrequencyCorrection(const int direction, const size_t channel) const
+{
+    //backwards compatibility with "CORR" string arg
+    const auto components = this->listFrequencies(direction, channel);
+    return (std::find(components.begin(), components.end(), "CORR") != components.end());
+}
+
+void SoapySDR::Device::setFrequencyCorrection(const int direction, const size_t channel, const double value)
+{
+    //backwards compatibility with "CORR" string arg
+    const auto components = this->listFrequencies(direction, channel);
+    if (std::find(components.begin(), components.end(), "CORR") != components.end())
+    {
+        this->setFrequency(direction, channel, "CORR", value);
+    }
+}
+
+double SoapySDR::Device::getFrequencyCorrection(const int direction, const size_t channel) const
+{
+    //backwards compatibility with "CORR" string arg
+    const auto components = this->listFrequencies(direction, channel);
+    if (std::find(components.begin(), components.end(), "CORR") != components.end())
+    {
+        return this->getFrequency(direction, channel, "CORR");
+    }
+    return 0.0;
+}
+
 /*******************************************************************
  * Gain API
  ******************************************************************/
@@ -480,6 +508,17 @@ std::vector<double> SoapySDR::Device::listSampleRates(const int, const size_t) c
     return std::vector<double>();
 }
 
+SoapySDR::RangeList SoapySDR::Device::getSampleRateRange(const int direction, const size_t channel) const
+{
+    SoapySDR::RangeList ranges;
+    //call into the older deprecated listSampleRates() call
+    for (auto &bw : this->listSampleRates(direction, channel))
+    {
+        ranges.push_back(SoapySDR::Range(bw, bw));
+    }
+    return ranges;
+}
+
 /*******************************************************************
  * Bandwidth API
  ******************************************************************/
@@ -642,6 +681,16 @@ unsigned SoapySDR::Device::readRegister(const unsigned) const
     return 0;
 }
 
+void SoapySDR::Device::writeRegisters(const std::string &, const unsigned, const std::vector<unsigned> &)
+{
+    return;
+}
+
+std::vector<unsigned> SoapySDR::Device::readRegisters(const std::string &, const unsigned, size_t length) const
+{
+    return std::vector<unsigned>(length, 0);
+}
+
 /*******************************************************************
  * Settings API
  ******************************************************************/
