@@ -45,6 +45,7 @@ int vasprintf(char **strp, const char *fmt, va_list ap)
     int r = _vscprintf(fmt, ap);
     if (r < 0) return r;
     *strp = (char *)malloc(r+1);
+    if (*strp == nullptr) return -1;
     return vsprintf_s(*strp, r+1, fmt, ap);
 }
 #endif
@@ -64,7 +65,7 @@ int vasprintf(char **strp, const char *fmt, va_list ap)
 /***********************************************************************
  * Default log message handler implementation
  **********************************************************************/
-void defaultLogHandler(const SoapySDRLogLevel logLevel, const char *message)
+static void defaultLogHandler(const SoapySDRLogLevel logLevel, const char *message)
 {
     switch (logLevel)
     {
@@ -102,9 +103,17 @@ void SoapySDR_vlogf(const SoapySDRLogLevel logLevel, const char *format, va_list
     }
 }
 
+/***********************************************************************
+ * Replace the current registeredLogHandler with handler.
+ * If nullptr is passed then the default log handler is restored.
+ **********************************************************************/
 void SoapySDR_registerLogHandler(const SoapySDRLogHandler handler)
 {
-    registeredLogHandler = handler;
+    if (handler) {
+        registeredLogHandler = handler;
+    } else {
+        registeredLogHandler = defaultLogHandler;
+    }
 }
 
 void SoapySDR_setLogLevel(const SoapySDRLogLevel logLevel)
