@@ -1,4 +1,4 @@
-// Copyright (c) 2014-2016 Josh Blum
+// Copyright (c) 2014-2018 Josh Blum
 // SPDX-License-Identifier: BSL-1.0
 
 #include <SoapySDR/Modules.hpp>
@@ -8,6 +8,7 @@
 #include <string>
 #include <cstdlib> //getenv
 #include <sstream>
+#include <mutex>
 #include <map>
 
 #ifdef _MSC_VER
@@ -17,6 +18,12 @@
 #include <glob.h>
 #endif
 
+static std::recursive_mutex &getModuleMutex(void)
+{
+    static std::recursive_mutex mutex;
+    return mutex;
+}
+
 /***********************************************************************
  * root installation path
  **********************************************************************/
@@ -73,11 +80,10 @@ std::string SoapySDR::getRootPath(void)
  **********************************************************************/
 static std::vector<std::string> searchModulePath(const std::string &path)
 {
-    const std::string pattern = path + "*.*";
     std::vector<std::string> modulePaths;
+    const std::string pattern = path + "*@MODULE_EXT@";
 
 #ifdef _MSC_VER
-
     //http://stackoverflow.com/questions/612097/how-can-i-get-a-list-of-files-in-a-directory-using-c-or-c
     WIN32_FIND_DATA fd; 
     HANDLE hFind = ::FindFirstFile(pattern.c_str(), &fd); 
@@ -96,7 +102,6 @@ static std::vector<std::string> searchModulePath(const std::string &path)
     }
 
 #else
-
     glob_t globResults;
 
     const int ret = glob(pattern.c_str(), 0/*no flags*/, NULL, &globResults);
@@ -163,6 +168,8 @@ std::vector<std::string> SoapySDR::listModules(void)
 
 std::vector<std::string> SoapySDR::listModules(const std::string &path)
 {
+    static const std::string suffix("@MODULE_EXT@");
+    if (path.rfind(suffix) == (path.size()-suffix.size())) return {path};
     return searchModulePath(path + "/"); //requires trailing slash
 }
 
@@ -189,6 +196,17 @@ std::map<std::string, SoapySDR::Kwargs> &getLoaderResults(void)
     return results;
 }
 
+std::map<std::string, std::string> &getModuleVersions(void)
+{
+    static std::map<std::string, std::string> versions;
+    return versions;
+}
+
+SoapySDR::ModuleVersion::ModuleVersion(const std::string &version)
+{
+    getModuleVersions()[getModuleLoading()] = version;
+}
+
 #ifdef _MSC_VER
 static std::string GetLastErrorMessage(void)
 {
@@ -211,8 +229,15 @@ static std::string GetLastErrorMessage(void)
 }
 #endif
 
+static bool enableAutomaticLoadModules(true);
+
 std::string SoapySDR::loadModule(const std::string &path)
 {
+    std::lock_guard<std::recursive_mutex> lock(getModuleMutex());
+
+    //disable automatic load modules when individual modules are manually loaded
+    enableAutomaticLoadModules = false;
+
     //check if already loaded
     if (getModuleHandles().count(path) != 0) return path + " already loaded";
 
@@ -243,12 +268,22 @@ std::string SoapySDR::loadModule(const std::string &path)
 
 SoapySDR::Kwargs SoapySDR::getLoaderResult(const std::string &path)
 {
+    std::lock_guard<std::recursive_mutex> lock(getModuleMutex());
     if (getLoaderResults().count(path) == 0) return SoapySDR::Kwargs();
     return getLoaderResults()[path];
 }
 
+std::string SoapySDR::getModuleVersion(const std::string &path)
+{
+    std::lock_guard<std::recursive_mutex> lock(getModuleMutex());
+    if (getModuleVersions().count(path) == 0) return "";
+    return getModuleVersions()[path];
+}
+
 std::string SoapySDR::unloadModule(const std::string &path)
 {
+    std::lock_guard<std::recursive_mutex> lock(getModuleMutex());
+
     //check if already loaded
     if (getModuleHandles().count(path) == 0) return path + " never loaded";
 
@@ -269,6 +304,7 @@ std::string SoapySDR::unloadModule(const std::string &path)
 
     //clear the handle
     getLoaderResults().erase(path);
+    getModuleVersions().erase(path);
     getModuleHandles().erase(path);
     return "";
 }
@@ -279,11 +315,29 @@ std::string SoapySDR::unloadModule(const std::string &path)
 
 void lateLoadNullDevice(void);
 
-void SoapySDR::loadModules(void)
+void automaticLoadModules(void)
 {
+    std::lock_guard<std::recursive_mutex> lock(getModuleMutex());
+
+    //loaded variable makes automatic load a one-shot
     static bool loaded = false;
     if (loaded) return;
     loaded = true;
+
+    //initialize any static units in the library
+    //rather than rely on static initialization
+    lateLoadNullDevice();
+
+    //load the modules when not otherwise disabled
+    if (enableAutomaticLoadModules) SoapySDR::loadModules();
+}
+
+void SoapySDR::loadModules(void)
+{
+    std::lock_guard<std::recursive_mutex> lock(getModuleMutex());
+
+    //initialize any static units in the library
+    //rather than rely on static initialization
     lateLoadNullDevice();
 
     const auto paths = listModules();
